//=============================================================================
// Demo program "Bluetooth Messaging with External BT Device".
//
// Can be run under control of the ROBO TX Controller
// firmware in download (local) mode.
// Starts and stops motor, connected to outputs M1, by means
// of the button, connected to the input I8 on other
// ROBO TX Controller. Pulses from the motor are calculated
// by the counter C1. The motor is stopped after the counter
// reaches the value of 1000.
//
// Disclaimer - Exclusion of Liability
//
// This software is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. It can be used and modified by anyone
// free of any license obligations or authoring rights.
//=============================================================================

#include "ROBO_TX_PRG.h"

#define MOTOR_NUMBER    1
#define MOTOR_IDX       (MOTOR_NUMBER - 1)

#define BT_CHANNEL      1

// Bluetooth address of external BT device (e.g. SmartPhone)
static UCHAR8 bt_addr[6] = {0x00,0x00,0x00,0x00,0x00,0x00};     // get it dynamically in PrgInit 
static UCHAR8 *bt_address = &bt_addr[0];

static enum
{
    START_LISTEN,
    PAUSE_1,
    WAIT_CONNECT,
    PAUSE_2,
    PAUSE_3,
    RECEIVE,
    PAUSE_4,
    EXIT
} stage;

static unsigned long timer;
static enum bt_commands_e command;
static CHAR8 command_status;
static CHAR8 receive_command_status;

static UCHAR8 keys[9] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };

/*-----------------------------------------------------------------------------
 * Function Name       : BtCallback
 *
 * This callback function is called to inform the program about result (status)
 * of execution of any Bluetooth command except BtStartReceive command.
 *-----------------------------------------------------------------------------*/
static void BtCallback
(
    TA * p_ta_array,
    BT_CB * p_data
)
{
    command_status = p_data->status;
}


/*-----------------------------------------------------------------------------
 * Function Name       : BtReceiveCallback
 *
 * This callback function is called to inform the program about result (status)
 * of execution of BtStartReceive command. It is also called when a message
 * arrives via Bluetooth.
 *-----------------------------------------------------------------------------*/
static void BtReceiveCallback
(
    TA * p_ta_array,
    BT_RECV_CB * p_data
)
{
    if (p_data->status == BT_MSG_INDICATION)
    {
        TA * p_ta = &p_ta_array[TA_LOCAL];
        
        UCHAR8 rxmsg[64];
        int rxlen=0;
        
        if(p_data->msg_len <= 63)
        {
            p_ta->hook_table.memcpy(rxmsg, p_data->msg, p_data->msg_len);
            rxlen = p_data->msg_len;
            rxmsg[p_data->msg_len] = 0;

            if(!p_ta->hook_table.strcmp((char *)rxmsg, "del_ebd"))
            {
                UCHAR8 bta_null[6] = {0x00,0x00,0x00,0x00,0x00,0x00};
                
                // reset BT address of registered external message device to 0
                p_ta->hook_table.SetExtBtDevAddr(bta_null);
                
                p_ta->hook_table.DisplayMsg(p_ta, "External Bluetooth Device: Reference Deleted");
            }    
        }    
        

        if(rxlen > 0)
        {
			// Copy key states
			p_ta->hook_table.memcpy(keys, rxmsg, rxlen);

            // Send BT message
            command_status = -1;
            p_ta->hook_table.BtSend(BT_CHANNEL, rxlen, rxmsg, BtCallback);
        }    
    }
    else
    {
        receive_command_status = p_data->status;
    }
}


/*-----------------------------------------------------------------------------
 * Function Name       : PrgInit
 *
 * This it the program initialization.
 * It is called once.
 *-----------------------------------------------------------------------------*/
void PrgInit
(
    TA * p_ta_array,    // pointer to the array of transfer areas
    int ta_count        // number of transfer areas in array (equal to TA_COUNT)
)
{
    TA * p_ta = &p_ta_array[TA_LOCAL];

    // get BT address of registered external message device
    p_ta->hook_table.GetExtBtDevAddr(bt_addr);

    // Start listen to the controller with bt_address via Bluetooth channel BT_CHANNEL
    stage = START_LISTEN;
    command = CMD_START_LISTEN;
    command_status = -1;
    p_ta->hook_table.BtStartListen(BT_CHANNEL, bt_address, BtCallback);
}


/*-----------------------------------------------------------------------------
 * Function Name       : PrgTic
 *
 * This is the main function of this program.
 * It is called every tic (1 ms) realtime.
 *-----------------------------------------------------------------------------*/
int PrgTic
(
    TA * p_ta_array,    // pointer to the array of transfer areas
    int ta_count        // number of transfer areas in array (equal to TA_COUNT)
)
{
    int rc = 0x7FFF; // return code: 0x7FFF - program should be further called by the firmware;
                     //              0      - program should be normally stopped by the firmware;
                     //              any other value is considered by the firmware as an error code
                     //              and the program is stopped.
    TA * p_ta = &p_ta_array[TA_LOCAL];

    switch (stage)
    {
        case START_LISTEN:
            if (command_status >= 0)
            {
                if (BtDisplayCommandStatus(p_ta, bt_address, BT_CHANNEL, command, command_status))
                {
                    if (command_status != BT_SUCCESS)
                    {
                        stage = PAUSE_4;
                    }
                    else
                    {
                        stage = PAUSE_1;
                        command_status = -1;
                    }
                    timer = 0;
                }
            }
            break;

        case PAUSE_1: // to let a user to notice the "Started listening" display output
            if (++timer >= 3000) // wait for 3 seconds
            {
                stage = WAIT_CONNECT;
            }
            else
            {
                break;
            }

        case WAIT_CONNECT:
            if (command_status >= 0)
            {
                if (BtDisplayCommandStatus(p_ta, bt_address, BT_CHANNEL, command, command_status))
                {
                    if (command_status == BT_CON_INDICATION)
                    {
                        stage = PAUSE_2;

                        // Start receive from Bluetooth channel BT_CHANNEL
                        command = CMD_START_RECEIVE;
                        receive_command_status = -1;
                        p_ta->hook_table.BtStartReceive(BT_CHANNEL, BtReceiveCallback);
                    }
                    command_status = -1;
                    timer = 0;
                }
            }
            break;

        case PAUSE_2: // to let a user to notice the "Passive connection establishment
                      // (incoming connection)" display output
            if (++timer >= 3000) // wait for 3 seconds
            {
                stage = PAUSE_3;
                timer = 0;
            }
            else
            {
                break;
            }

        case PAUSE_3: // to let a user to notice the "Started receiving" display output
            if (receive_command_status >= 0)
            {
                if (BtDisplayCommandStatus(p_ta, bt_address, BT_CHANNEL, command, receive_command_status))
                {
                    if (receive_command_status == BT_SUCCESS)
                    {
                        // Successful result of BtStartReceive() - restart the timer to let a user to
                        // notice this display output
                        receive_command_status = -1;
                        timer = 0;
                    }
                    else
                    {
                        // Unsuccessful result of BtStartReceive() - make PAUSE_4 (to let a user to
                        // notice this display output) and exit the program
                        stage = PAUSE_4;
                        timer = 0;
                    }
                }
                break;
            }
            else if (++timer >= 3000) // wait for 3 seconds
            {
                if (!p_ta->hook_table.IsDisplayBeingRefreshed(p_ta)) // wait until display is refreshed
                {
                    // Drop all pop-up messages from display and return to the main frame
                    p_ta->hook_table.DisplayMsg(p_ta, NULL);

                    stage = RECEIVE;
                }
            }
            else
            {
                break;
            }

        case RECEIVE:
            if (command_status >= 0)
            {
                if (command_status != BT_SUCCESS)
                {
                    // Usually if we have come here, then this means the other controller
                    // has disconnected from us

                    /*
                    // This block of code should be used if we want that the program stops when other
                    // ROBO TX Controller disconnects from us
                    if (BtDisplayCommandStatus(p_ta, bt_address, BT_CHANNEL, command, command_status))
                    {
                        stage = PAUSE_4;
                        timer = 0;
                    }
                    */
                }
            }
			
			/* EXAMPLE */
			/*  */ if (keys[3] == 1) {
			/*  */     p_ta->output.duty[0] = DUTY_MAX;
			/*  */     p_ta->output.duty[1] = 0;
			/*  */ 
			/*  */ } else if (keys[5] == 1) {
			/*  */     p_ta->output.duty[0] = 0;
			/*  */     p_ta->output.duty[1] = DUTY_MAX;
			/*  */ 
			/*  */ } else {
			/*  */     p_ta->output.duty[0] = 0;
			/*  */     p_ta->output.duty[1] = 0;
			/*  */ }
			/*  */ 
			/*  */ if (keys[1] == 1) {
			/*  */     p_ta->output.duty[2] = DUTY_MAX;
			/*  */     p_ta->output.duty[3] = 0;
			/*  */ 
			/*  */ } else if (keys[7] == 1) {
			/*  */     p_ta->output.duty[2] = 0;
			/*  */     p_ta->output.duty[3] = DUTY_MAX;
			/*  */ 
			/*  */ } else {
			/*  */     p_ta->output.duty[2] = 0;
			/*  */     p_ta->output.duty[3] = 0;
			/*  */ }
			/*  */ 
			/*  */ if (keys[0] == 1) {
			/*  */     p_ta->output.duty[4] = DUTY_MAX;
			/*  */     p_ta->output.duty[5] = 0;
			/*  */ 
			/*  */ } else if (keys[6] == 1) {
			/*  */     p_ta->output.duty[4] = 0;
			/*  */     p_ta->output.duty[5] = DUTY_MAX;
			/*  */ 
			/*  */ } else {
			/*  */     p_ta->output.duty[4] = 0;
			/*  */     p_ta->output.duty[5] = 0;
			/*  */ }
			/*  */ 
			/*  */ if (keys[2] == 1) {
			/*  */     p_ta->output.duty[6] = DUTY_MAX;
			/*  */     p_ta->output.duty[7] = 0;
			/*  */ 
			/*  */ } else if (keys[8] == 1) {
			/*  */     p_ta->output.duty[6] = 0;
			/*  */     p_ta->output.duty[7] = DUTY_MAX;
			/*  */ 
			/*  */ } else {
			/*  */     p_ta->output.duty[6] = 0;
			/*  */     p_ta->output.duty[7] = 0;
			/*  */ }
			/* EXAMPLE */
			
            break;

        case PAUSE_4: // to let a user to notice the last display output
            if (++timer >= 3000) // wait for 3 seconds
            {
                stage = EXIT;
            }
            else
            {
                break;
            }

        case EXIT:
            if (!p_ta->hook_table.IsDisplayBeingRefreshed(p_ta)) // wait until display is refreshed
            {
                rc = 0; // stop program
            }
            break;

        default:
            break;
    }
    return rc;
}
